// --------------------------------------
// i2c_scanner
//
// Version 1
//    This program (or code that looks like it)
//    can be found in many places.
//    For example on the Arduino.cc forum.
//    The original author is not know.
// Version 2, Juni 2012, Using Arduino 1.0.1
//     Adapted to be as simple as possible by Arduino.cc user Krodal
// Version 3, Feb 26  2013
//    V3 by louarnold
// Version 4, March 3, 2013, Using Arduino 1.0.3
//    by Arduino.cc user Krodal.
//    Changes by louarnold removed.
//    Scanning addresses changed from 0...127 to 1...119,
//    according to the i2c scanner by Nick Gammon
//    http://www.gammon.com.au/forum/?id=10896
// Version 5, March 28, 2013
//    As version 4, but address scans now to 127.
//    A sensor seems to use address 120.
// Version 6, Dec 29, 2013
//    reduced size (RAM, Flash)
// Version 7, Dec 30, 2013
//    Added displaying the content of known devices
// Version 8, Dec 31, 2013
//    Added PCF8574A/PCF8574T
//
//
// This sketch tests the standard 7-bit addresses
// Devices with higher bit address might not be seen properly.
//

#include <Wire.h>

void PrintAddress(byte addr, byte details)
{
  if ( ( details == 1 ) or ( details == 5 ) )
  {
    Serial.print(F(" at address 0x"));
  };
  if ( details == 3 )
  {
    Serial.print(F("(0x"));
  };
  if ( details == 4 )
  {
    Serial.print(F("0x"));
  };
  if ( addr < 16 )
  {
    Serial.print("0");
  };
  Serial.print(addr, HEX);
  if ( details == 1 )
  {
    Serial.print(" (");
    Serial.print(addr);
    Serial.println(")");
  };
  if ( details == 2 )
  {
    Serial.print(" ");
    Serial.print(addr);
    Serial.print(" ");
    Serial.println((char)addr);
  };
  if ( details == 3 )
  {
    Serial.println(")");
  };
  if ( details == 4 )
  {
    Serial.print(" ");
  };
  if ( details == 5 )
  {
    Serial.print(" (");
    Serial.print(addr);
    Serial.print(") ");
  };
};

void PrintBinary(byte _Value)
{
  Serial.print(F(" attempt:  0b"));
  for (int index = 128; index >= 1; index = index >> 1) {
    if ( (index & _Value) == index ) {
      Serial.print("1");
    } else {
      Serial.print("0");
    };
  };
  Serial.println();
};  

void LCD_bl_test(byte _i2cAddr, byte _i2cValueON, byte _i2cValueOFF)
{

  Serial.print(F("LCD backlight OFF"));
  PrintBinary(_i2cValueOFF);
  Wire.beginTransmission(_i2cAddr);
  Wire.requestFrom( _i2cAddr, (uint8_t)1 );
  while (!Wire.available())
  {
    // waiting
  }
  Wire.write(_i2cValueOFF);
  Wire.endTransmission();
  delay(2000);

  Serial.print(F("LCD backlight  ON"));
  PrintBinary(_i2cValueON);
  Wire.beginTransmission(_i2cAddr);
  Wire.requestFrom( _i2cAddr, (uint8_t)1 );
  while (!Wire.available())
  {
    // waiting
  }
  Wire.write(_i2cValueON);
  Wire.endTransmission();
  delay(2000);
};

byte bcdToDec(byte val)
{
  return ( (val / 16 * 10) + (val % 16) );
};

void RTCDump(byte DS3231_ADDRESS, unsigned int NumberOfBytes)
{
  Serial.print(F("Dump of NVRAM content ("));
  Serial.print(NumberOfBytes);
  Serial.print(F(" bytes) from I2C device"));
  PrintAddress(DS3231_ADDRESS, 1);
  Wire.beginTransmission(DS3231_ADDRESS);
  Wire.write(0);
  Wire.endTransmission();
  if ( Wire.requestFrom(DS3231_ADDRESS, NumberOfBytes) == NumberOfBytes ) {
    while (!Wire.available())
    {
      // waiting
    }
    byte addr = 0;
    Serial.println(F("The information is apply only for chip DS3231, partially for chips DS1337 and DS1307 (and maybe for another similar chips)"));
    PrintAddress(addr++, 4); Serial.print(F("Seconds             = ")); Serial.println(bcdToDec(Wire.read() & 0x7F));
    PrintAddress(addr++, 4); Serial.print(F("Minutes             = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Hours               = ")); Serial.println(bcdToDec(Wire.read() & 0x3F));
    PrintAddress(addr++, 4); Serial.print(F("Weekday             = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Day                 = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Month/Century       = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Year                = ")); Serial.println(bcdToDec(Wire.read()) + 2000);
    PrintAddress(addr++, 4); Serial.print(F("Alarm 1 Seconds     = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Alarm 1 Minutes     = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Alarm 1 Hours       = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Alarm 1 Day&Date    = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Alarm 2 Minutes     = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Alarm 2 Hours       = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Alarm 2 Day&Date    = ")); Serial.println(bcdToDec(Wire.read()));
    PrintAddress(addr++, 4); Serial.print(F("Control             = ")); Serial.println(Wire.read(), BIN);
    PrintAddress(addr++, 4); Serial.print(F("Control/Status      = ")); Serial.println(Wire.read(), BIN);
    PrintAddress(addr++, 4); Serial.print(F("Aging Offset        = ")); Serial.println(Wire.read());
    PrintAddress(addr++, 4); Serial.print(F("Temperature MSB+LSB = "));
    //MSB of Temp
    float ttc = (float)(int)Wire.read();
    //LSB of Temp
    addr++;
    byte portion = Wire.read();
    if (portion == 0b01000000) ttc += 0.25;
    if (portion == 0b10000000) ttc += 0.5;
    if (portion == 0b11000000) ttc += 0.75;
    char buffer[16];
    String TempStr = dtostrf(ttc, 5, 2, buffer);
    TempStr = TempStr + "\xB0\x43"; //oC for Western Character Set
    Serial.println(TempStr);
  }
  else {
    Serial.println(F("Error"));
  };
  Serial.println(F("End of Dump\n"));
};

byte i2c_eeprom_read_byte( int deviceaddress, unsigned int eeaddress )
{
  byte rdata = 0xFF;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress, 1);
  if (Wire.available()) rdata = Wire.read();
  return rdata;
};

void MemoryDump(byte I2C_EEPROM_ADDRESS, unsigned int NumberOfBytes)
{
  Serial.print(F("Dump of memory content ("));
  Serial.print(NumberOfBytes);
  Serial.print(F(" bytes) from I2C device"));
  PrintAddress(I2C_EEPROM_ADDRESS, 1);
  unsigned int addr = 0;
  byte b = i2c_eeprom_read_byte(I2C_EEPROM_ADDRESS, addr);
  char c;
  while (addr < NumberOfBytes)
  {
    c = (char)b;
    Serial.print(c); //print content to serial port
    if ( (b < 0x20) or (b >= 0x7F) )
    {
      PrintAddress(b, 3);
    };
    addr++; //increase address
    b = i2c_eeprom_read_byte(I2C_EEPROM_ADDRESS, addr);
  };
  Serial.println(F("\nEnd of Dump\n"));
};

void setup()
{
  delay(2000); //wait for terminal to be open by user
  Wire.begin();
  Serial.begin(115200);
  Serial.println(F("\nI2C Scanner\n"));
};

void loop()
{
  byte error, address;
  int nDevices;
  int aDevices[125];

  Serial.println(F("Scanning...\n"));

  nDevices = 0;
  for ( address = 0; address < 126; address++ )
  {
    aDevices[address] = 0;
  };
  for ( address = 1; address < 127; address++ )
  {
    // The i2c_scanner uses the return value of
    // the Write.endTransmisstion to see if
    // a device did acknowledge to the address.
    Wire.beginTransmission(address);
    error = Wire.endTransmission();

    if ( error == 0 )
    {
      Serial.print(F("I2C device found"));
      PrintAddress(address, 5);
      aDevices[nDevices] = address;
      nDevices++;
      if ( ( address >= 0x20 ) and ( address <= 0x27 ) )
      {
        Serial.print(F("it could be a LCD Backpack (PCF8574x)"));
      };
      if ( ( address >= 0x38 ) and ( address <= 0x3F ) )
      {
        Serial.print(F("it could be a LCD Backpack (PCF8574A)"));
      };
      if ( ( address >= 0x50 ) and ( address <= 0x57 ) )
      {
        Serial.print(F("it could be a EEPROM AT24Cxxxx"));
      };
      if ( address == 0x68 )
      {
        Serial.print(F("it could be a Real Time Clock DS3231/DS1337/DS1307/..."));
      };
      Serial.println();
    }
    else if ( error == 4 )
    {
      Serial.print(F("Unknown error"));
      PrintAddress(address, 1);
    };
    Wire.endTransmission();
  };
  if ( nDevices == 0 )
  {
    Serial.println(F("No I2C devices found\n"));
  }
  else
  {
    Serial.println(F("\nDone!\n"));
  };

  //DUMP DEVICES
  for ( address = 0; address < 126; address++ )
  {
    if ( aDevices[address] != 0 )
    {
      //LCD backapack
      if ( ( aDevices[address] >= 0x20 ) and ( aDevices[address] <= 0x27 ) )
      {
        LCD_bl_test( aDevices[address], 0b00000000, 0b10000000 );
        LCD_bl_test( aDevices[address], 0b00001000, 0b00000000 );
        Serial.println(F("End of LCD test\n"));
      };
      if ( ( aDevices[address] >= 0x38 ) and ( aDevices[address] <= 0x3F ) )
      {
        LCD_bl_test( aDevices[address], 0b00000001, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b00000010, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b00000100, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b00001000, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b00010000, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b00100000, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b01000000, 0b00000000 );
        LCD_bl_test( aDevices[address], 0b10000000, 0b00000000 );
        Serial.println(F("End of LCD test\n"));
      };
      //AT24Cxxxx EEPROM
      if ( ( aDevices[address] >= 0x50 ) and ( aDevices[address] <= 0x57 ) )
      {
        MemoryDump( aDevices[address], 111 );
      };
      //DSxxxx RTC
      if ( aDevices[address] == 0x68 )
      {
        RTCDump( aDevices[address], 19 );
      };
    };
  };
  Serial.println(F("Send any key to repeat..."));
  while ( Serial.available() == 0 );
  while ( Serial.available() > 0 )
  {
    Serial.read();
  };
};

